//-----------------------------------------------------------------------------
// File: MouseExc.cpp
//
// Desc: Demonstrates an application which receives relative mouse data
//       in exclusive mode via a dialog timer.
//
// Copyright (c) 1998-1999 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

//#define INITGUID
#include "inputlib.h"
#include "..\gamelog\log.h"
LOG inputlog("input.log");

//#include "resource.h"

//-----------------------------------------------------------------------------
// Global variables for the DirectMusic sample 
//-----------------------------------------------------------------------------
LPDIRECTINPUT7       g_pDI              = NULL;         
//IDirectInput*           g_pDI       = NULL;
IDirectInputDevice*     g_pMouse    = NULL;

//HINSTANCE               g_hInst     = NULL;
BOOL                    g_bActive   = TRUE;

LPDIRECTINPUTDEVICE g_pKeyboard = NULL;

LPDIRECTINPUTDEVICE2 g_pJoystick        = NULL;     
DIDEVCAPS            g_diDevCaps;



//-----------------------------------------------------------------------------
// Name: EnumJoysticksCallback()
// Desc: Called once for each enumerated joystick. If we find one, create a
//       device interface on it so we can play with it.
//-----------------------------------------------------------------------------
BOOL CALLBACK EnumJoysticksCallback( const DIDEVICEINSTANCE* pdidInstance,
                                     VOID* pContext )
{
    HRESULT hr;

    // Obtain an interface to the enumerated joystick.
    hr = g_pDI->CreateDeviceEx( pdidInstance->guidInstance, IID_IDirectInputDevice2,
		                        (VOID**)&g_pJoystick, NULL );

    // If it failed, then we can't use this joystick. (Maybe the user unplugged
    // it while we were in the middle of enumerating it.)
    if( FAILED(hr) ) 
        return DIENUM_CONTINUE;


    // Stop enumeration. Note: we're just taking the first joystick we get. You
    // could store all the enumerated joysticks and let the user pick.
    return DIENUM_STOP;
}

//-----------------------------------------------------------------------------
// Name: EnumAxesCallback()
// Desc: Callback function for enumerating the axes on a joystick
//-----------------------------------------------------------------------------
BOOL CALLBACK EnumAxesCallback( const DIDEVICEOBJECTINSTANCE* pdidoi,
                                VOID* pContext )
{
    HWND hDlg = (HWND)pContext;

    DIPROPRANGE diprg; 
    diprg.diph.dwSize       = sizeof(DIPROPRANGE); 
    diprg.diph.dwHeaderSize = sizeof(DIPROPHEADER); 
    diprg.diph.dwHow        = DIPH_BYOFFSET; 
    diprg.diph.dwObj        = pdidoi->dwOfs; // Specify the enumerated axis
    diprg.lMin              = -1000; 
    diprg.lMax              = +1000; 
    
    // Set the range for the axis
    if( FAILED( g_pJoystick->SetProperty( DIPROP_RANGE, &diprg.diph ) ) )
        return DIENUM_STOP;

    /*
	// Set the UI to reflect what axes the joystick supports
    switch( pdidoi->dwOfs )
    {
        case DIJOFS_X:
            EnableWindow( GetDlgItem( hDlg, IDC_X_AXIS ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_X_AXIS_TEXT ), TRUE );
            break;
        case DIJOFS_Y:
            EnableWindow( GetDlgItem( hDlg, IDC_Y_AXIS ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_Y_AXIS_TEXT ), TRUE );
            break;
        case DIJOFS_Z:
            EnableWindow( GetDlgItem( hDlg, IDC_Z_AXIS ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_Z_AXIS_TEXT ), TRUE );
            break;
        case DIJOFS_RX:
            EnableWindow( GetDlgItem( hDlg, IDC_X_ROT ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_X_ROT_TEXT ), TRUE );
            break;
        case DIJOFS_RY:
            EnableWindow( GetDlgItem( hDlg, IDC_Y_ROT ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_Y_ROT_TEXT ), TRUE );
            break;
        case DIJOFS_RZ:
            EnableWindow( GetDlgItem( hDlg, IDC_Z_ROT ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_Z_ROT_TEXT ), TRUE );
            break;
        case DIJOFS_SLIDER(0):
            EnableWindow( GetDlgItem( hDlg, IDC_SLIDER0 ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_SLIDER0_TEXT ), TRUE );
            break;
        case DIJOFS_SLIDER(1):
            EnableWindow( GetDlgItem( hDlg, IDC_SLIDER1 ), TRUE );
            EnableWindow( GetDlgItem( hDlg, IDC_SLIDER1_TEXT ), TRUE );
            break;
    }
	*/
    return DIENUM_CONTINUE;
}

//-----------------------------------------------------------------------------
// Function: InitDirectInput
//
// Description: 
//      Initialize the DirectInput variables.
//
//-----------------------------------------------------------------------------
HRESULT InitDirectInput(HINSTANCE hinsT, HWND hDlg )
{
    HRESULT hr;
	inputlog.addlog("Setting up direct input");	

    // Register with the DirectInput subsystem and get a pointer
    // to a IDirectInput interface we can use.
    hr = DirectInputCreateEx( hinsT, DIRECTINPUT_VERSION,IID_IDirectInput7, (LPVOID*)&g_pDI, NULL );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to create direct input");	
        return hr;
	}
    //hr = DirectInputCreate( hinsT, DIRECTINPUT_VERSION, &g_pDI, NULL );
    //if ( FAILED(hr) ) 
    //    return hr;

    // Obtain an interface to the system mouse device.
    hr = g_pDI->CreateDevice( GUID_SysMouse, &g_pMouse, NULL );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to create mouse device");	
        return hr;
	}

    // Set the data format to "mouse format" - a predefined data format 
    //
    // A data format specifies which controls on a device we
    // are interested in, and how they should be reported.
    //
    // This tells DirectInput that we will be passing a
    // DIMOUSESTATE structure to IDirectInputDevice::GetDeviceState.
    hr = g_pMouse->SetDataFormat( &c_dfDIMouse );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to set mouse data format");	
        return hr;
	}

    // Set the cooperativity level to let DirectInput know how
    // this device should interact with the system and with other
    // DirectInput applications.
    hr = g_pMouse->SetCooperativeLevel( hDlg, 
                                        DISCL_EXCLUSIVE | DISCL_FOREGROUND);
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to set mouse cooperative level");	
        return hr;
	}

	// Obtain an interface to the system keyboard device.
	hr = g_pDI->CreateDevice( GUID_SysKeyboard, &g_pKeyboard, NULL );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to create keyboard interface");	
        return hr;
	}

	// Set the data format to "keyboard format" - a predefined data format
	//
	// A data format specifies which controls on a device we
	// are interested in, and how they should be reported.
	//
	// This tells DirectInput that we will be passing an array
	// of 256 bytes to IDirectInputDevice::GetDeviceState.
	hr = g_pKeyboard->SetDataFormat( &c_dfDIKeyboard );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to set keyboard format");	
        return hr;
	}

	// Set the cooperativity level to let DirectInput know how
	// this device should interact with the system and with other
	// DirectInput applications.
	hr = g_pKeyboard->SetCooperativeLevel( hDlg,
									 DISCL_NONEXCLUSIVE | DISCL_FOREGROUND );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to set keyboard cooperative level");	
        return hr;
	}

	//add the joystick
    // Look for a simple joystick we can use for this sample program.
    hr = g_pDI->EnumDevices( DIDEVTYPE_JOYSTICK, EnumJoysticksCallback,
                             NULL, DIEDFL_ATTACHEDONLY );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to find joystick");	
        return hr;
	}

    
	// Make sure we got a joystick
    if( NULL == g_pJoystick )
    {
		inputlog.addlog("Failed to find joystick");	
        return E_FAIL;
    }

    // Set the data format to "simple joystick" - a predefined data format 
    //
    // A data format specifies which controls on a device we are interested in,
    // and how they should be reported. This tells DInput that we will be
    // passing a DIJOYSTATE structure to IDirectInputDevice::GetDeviceState().
    hr = g_pJoystick->SetDataFormat( &c_dfDIJoystick );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to set joystick format");	
        return hr;
	}

    // Set the cooperative level to let DInput know how this device should
    // interact with the system and with other DInput applications.
    hr = g_pJoystick->SetCooperativeLevel( hDlg, DISCL_EXCLUSIVE|DISCL_FOREGROUND );
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to set joystick cooperative level");	
        return hr;
	}

    // Determine how many axis the joystick has (so we don't error out setting
    // properties for unavailable axis)
    g_diDevCaps.dwSize = sizeof(DIDEVCAPS);
    hr = g_pJoystick->GetCapabilities(&g_diDevCaps);
    if( FAILED(hr) ) {
		inputlog.addlog("Failed to deterimine Joystick axis");	
        return hr;
	}
	

    // Enumerate the axes of the joyctick and set the range of each axis. Note:
    // we could just use the defaults, but we're just trying to show an example
    // of enumerating device objects (axes, buttons, etc.).

	//g_pJoystick->EnumObjects( EnumAxesCallback, (VOID*)g_pJoystick, DIDFT_AXIS );
    g_pJoystick->EnumObjects( EnumAxesCallback, (VOID*)hDlg, DIDFT_AXIS );


    return S_OK;
}

//-----------------------------------------------------------------------------
// Function: SetAcquire
//
// Description: 
//      Acquire or unacquire the mouse, depending on if the app is active
//       Input device must be acquired before the GetDeviceState is called
//
//-----------------------------------------------------------------------------
HRESULT SetAcquire()
{
    char szText[128];
    HWND hDlgText;

    // nothing to do if g_pMouse is NULL
    //****************************
	//if not initialized
	//if (NULL == g_pMouse)
    //    return S_FALSE;

    if (g_bActive) 
    {
        // acquire the input device 
		if (g_pMouse) g_pMouse->Acquire();
		if (g_pKeyboard) g_pKeyboard->Acquire();
        if (g_pJoystick) g_pJoystick->Acquire();

    } 
    else 
    {
        // update the dialog text 
        //strcpy( szText, "Unacquired" );
        //hDlgText = GetDlgItem( hDlg, IDC_MOUSE_STATE );
        //SetWindowText( hDlgText, szText );

        // unacquire the input device 
        if (g_pMouse) g_pMouse->Unacquire();
		if (g_pKeyboard) g_pKeyboard->Unacquire();
        if (g_pJoystick) g_pJoystick->Unacquire();
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Function: UpdateInputState
//
// Description: 
//      Get the input device's state and display it.
//
//-----------------------------------------------------------------------------
HRESULT UpdateInputState(dxinputstate *dinstate)
{
    //char szOldText[128];    // previous mouse state text
    //char szNewText[128];    // current  mouse state text
    HWND hDlgText;          // handle to static text box
	HRESULT hr;

    if (g_pMouse) 
    {
        //DIMOUSESTATE dims;      // DirectInput mouse state structure

        hr = DIERR_INPUTLOST;

        // if input is lost then acquire and keep trying 
        while ( DIERR_INPUTLOST == hr ) 
        {
            // get the input's device state, and put the state in dims
            hr = g_pMouse->GetDeviceState( sizeof(DIMOUSESTATE), &(dinstate->dimouse));

            if ( hr == DIERR_INPUTLOST )
            {
                // DirectInput is telling us that the input stream has
                // been interrupted.  We aren't tracking any state
                // between polls, so we don't have any special reset
                // that needs to be done.  We just re-acquire and
                // try again.
                hr = g_pMouse->Acquire();
                if ( FAILED(hr) )  
                    return hr;
            }
        }

		//if( FAILED(hr) ) {
		//	inputlog.addlog("Trouble reading mouse");	
		//	return hr;
		//}
		
        // The dims structure now has the state of the mouse, so 
        // display mouse coordinates (x, y, z) and buttons.
        //wsprintf( szNewText, "(%d, %d, %d) %c %c %c %c",
        //             dims.lX, dims.lY, dims.lZ,
        //             (dims.rgbButtons[0] & 0x80) ? '0' : ' ',
        //             (dims.rgbButtons[1] & 0x80) ? '1' : ' ',
        //             (dims.rgbButtons[2] & 0x80) ? '2' : ' ',
        //             (dims.rgbButtons[3] & 0x80) ? '3' : ' ');

        // if anything changed then repaint - avoid flicker
        //hDlgText = GetDlgItem( hDlg, IDC_MOUSE_STATE );
        //GetWindowText( hDlgText, szOldText, 255 );

        //if ( 0 != lstrcmp( szOldText, szNewText ) ) 
        //{
            // set the text on the dialog
        //    SetWindowText( hDlgText, szNewText );
        //}


    }

	if(g_pKeyboard)
	{
		//BYTE    diks[256];   // DirectInput keyboard state buffer

		// Get the input's device state, and put the state in dims
		hr = g_pKeyboard->GetDeviceState( sizeof(dinstate->diks), &(dinstate->diks) );
		if( FAILED(hr) )
		{
			// DirectInput is telling us that the input stream has been
			// interrupted.  We aren't tracking any state between polls, so
			// we don't have any special reset that needs to be done.
			// We just re-acquire and try again.

			// If input is lost then acquire and keep trying
			hr = g_pKeyboard->Acquire();
			while( hr == DIERR_INPUTLOST ||
				   hr == E_ACCESSDENIED  )
			{
				hr = g_pKeyboard->Acquire();
			}
			if( FAILED(hr) )
				return hr;
		}

/*		// Make a string of the index values of the keys that are down
		CHAR strNewText[256*3 + 1] = "";
		CHAR strLetter[4];

		for( int i = 0; i < 256; i++ )
		{
			  if( diks[i] & 0x80 )
			  {
				  sprintf( strLetter, "%02x ", i );
				  strcat( strNewText, strLetter );
			  }
		}

		// Get the old text in the text box
		CHAR strOldText[128];
		GetWindowText( GetDlgItem( hDlg, IDC_KEYBD_STATE ), strOldText, 127 );

		// If anything changed then repaint - avoid flicker
		if( 0 != lstrcmp( strOldText, strNewText ) )
			SetWindowText( GetDlgItem( hDlg, IDC_KEYBD_STATE ), strNewText );
*/
	}

	if(g_pJoystick)
	{
        do
        {
            // Poll the device to read the current state
            hr = g_pJoystick->Poll();
            if( FAILED(hr) )
                return hr;

            // Get the input's device state
            hr = g_pJoystick->GetDeviceState( sizeof(DIJOYSTATE), &dinstate->js );

            if( hr == DIERR_INPUTLOST )
            {
                // DInput is telling us that the input stream has been
                // interrupted. We aren't tracking any state between polls, so
                // we don't have any special reset that needs to be done. We
                // just re-acquire and try again.
                hr = g_pJoystick->Acquire();
                if( FAILED(hr) )  
                    return hr;
            }
        }
        while( DIERR_INPUTLOST == hr );

        if( FAILED(hr) )  
            return hr;
	}

    return S_OK;
}




//-----------------------------------------------------------------------------
// Function: FreeDirectInput
//
// Description: 
//      Initialize the DirectInput variables.
//
//-----------------------------------------------------------------------------
HRESULT FreeDirectInput()
{
	inputlog.addlog("Shutting down direct input");	
    // Unacquire and release any DirectInputDevice objects.
    if (NULL != g_pMouse) 
    {
        // Unacquire the device one last time just in case 
        // the app tried to exit while the device is still acquired.
        g_pMouse->Unacquire();

        g_pMouse->Release();
        g_pMouse = NULL;
    }

    // Unacquire and release any DirectInputDevice objects.
    if (NULL != g_pKeyboard) 
    {
        // Unacquire the device one last time just in case 
        // the app tried to exit while the device is still acquired.
        g_pKeyboard->Unacquire();

        g_pKeyboard->Release();
        g_pKeyboard = NULL;
    }
    // Unacquire and release any DirectInputDevice objects.
    if( NULL != g_pJoystick ) 
    {
        // Unacquire the device one last time just in case 
        // the app tried to exit while the device is still acquired.
        g_pJoystick->Unacquire();
        g_pJoystick->Release();
        g_pJoystick = NULL;
    }

    // Release any DirectInput objects.
    if (NULL != g_pDI) 
    {
        g_pDI->Release();
        g_pDI = NULL;
    }

    return S_OK;
}


//-----------------------------------------------------------------------------
// Function: FillKeyString
//
// Description: 
//    fills up a char[256] with printable codes corresponding to dx   
//    input state codes UPPER CASE
//
//-----------------------------------------------------------------------------

void FillKeyString(char *dxkeys) {
	for (int i=0; i<256; i++) dxkeys[i]=0;
	dxkeys[DXKEY_A]='A';
	dxkeys[DXKEY_B]='B';
	dxkeys[DXKEY_C]='C';
	dxkeys[DXKEY_D]='D';
	dxkeys[DXKEY_E]='E';
	dxkeys[DXKEY_F]='F';
	dxkeys[DXKEY_G]='G';
	dxkeys[DXKEY_H]='H';
	dxkeys[DXKEY_I]='I';
	dxkeys[DXKEY_J]='J';
	dxkeys[DXKEY_K]='K';
	dxkeys[DXKEY_L]='L';
	dxkeys[DXKEY_M]='M';
	dxkeys[DXKEY_N]='N';
	dxkeys[DXKEY_O]='O';
	dxkeys[DXKEY_P]='P';
	dxkeys[DXKEY_Q]='Q';
	dxkeys[DXKEY_R]='R';
	dxkeys[DXKEY_S]='S';
	dxkeys[DXKEY_T]='T';
	dxkeys[DXKEY_U]='U';
	dxkeys[DXKEY_V]='V';
	dxkeys[DXKEY_W]='W';
	dxkeys[DXKEY_X]='X';
	dxkeys[DXKEY_Y]='Y';
	dxkeys[DXKEY_Z]='Z';
	dxkeys[DXKEY_1]='!';
	dxkeys[DXKEY_2]='@';
	dxkeys[DXKEY_3]='#';
	dxkeys[DXKEY_4]='$';
	dxkeys[DXKEY_5]='%';
	dxkeys[DXKEY_6]='^';
	dxkeys[DXKEY_7]='&';
	dxkeys[DXKEY_8]='*';
	dxkeys[DXKEY_9]='(';
	dxkeys[DXKEY_0]=')';
	dxkeys[DXKEY_PAD1]='1';
	dxkeys[DXKEY_PAD2]='2';
	dxkeys[DXKEY_PAD3]='3';
	dxkeys[DXKEY_PAD4]='4';
	dxkeys[DXKEY_PAD5]='5';
	dxkeys[DXKEY_PAD6]='6';
	dxkeys[DXKEY_PAD7]='7';
	dxkeys[DXKEY_PAD8]='8';
	dxkeys[DXKEY_PAD9]='9';
	dxkeys[DXKEY_PAD0]='0';
	dxkeys[DXKEY_MINUS]='_';
	dxkeys[DXKEY_SPACE]=' ';
	dxkeys[DXKEY_FSLASH]='/';
	dxkeys[DXKEY_PAD_SLASH]='/';
	dxkeys[DXKEY_BSLASH]='\\';
	dxkeys[DXKEY_BSLASH2]='\\';
	dxkeys[DXKEY_PERIOD]='.';
}

//-----------------------------------------------------------------------------
// Function: FillKeyStringLower
//
// Description: 
//    fills up a char[256] with printable codes corresponding to dx   
//    input state codes Lower CASE
//
//-----------------------------------------------------------------------------

void FillKeyStringLower(char *dxkeys) {
	for (int i=0; i<256; i++) dxkeys[i]=0;
	dxkeys[DXKEY_A]='a';
	dxkeys[DXKEY_B]='b';
	dxkeys[DXKEY_C]='c';
	dxkeys[DXKEY_D]='d';
	dxkeys[DXKEY_E]='e';
	dxkeys[DXKEY_F]='f';
	dxkeys[DXKEY_G]='g';
	dxkeys[DXKEY_H]='h';
	dxkeys[DXKEY_I]='i';
	dxkeys[DXKEY_J]='j';
	dxkeys[DXKEY_K]='k';
	dxkeys[DXKEY_L]='l';
	dxkeys[DXKEY_M]='m';
	dxkeys[DXKEY_N]='n';
	dxkeys[DXKEY_O]='o';
	dxkeys[DXKEY_P]='p';
	dxkeys[DXKEY_Q]='q';
	dxkeys[DXKEY_R]='r';
	dxkeys[DXKEY_S]='s';
	dxkeys[DXKEY_T]='t';
	dxkeys[DXKEY_U]='u';
	dxkeys[DXKEY_V]='v';
	dxkeys[DXKEY_W]='w';
	dxkeys[DXKEY_X]='x';
	dxkeys[DXKEY_Y]='y';
	dxkeys[DXKEY_Z]='z';
	dxkeys[DXKEY_1]='1';
	dxkeys[DXKEY_2]='2';
	dxkeys[DXKEY_3]='3';
	dxkeys[DXKEY_4]='4';
	dxkeys[DXKEY_5]='5';
	dxkeys[DXKEY_6]='6';
	dxkeys[DXKEY_7]='7';
	dxkeys[DXKEY_8]='8';
	dxkeys[DXKEY_9]='9';
	dxkeys[DXKEY_0]='0';
	dxkeys[DXKEY_PAD1]='1';
	dxkeys[DXKEY_PAD2]='2';
	dxkeys[DXKEY_PAD3]='3';
	dxkeys[DXKEY_PAD4]='4';
	dxkeys[DXKEY_PAD5]='5';
	dxkeys[DXKEY_PAD6]='6';
	dxkeys[DXKEY_PAD7]='7';
	dxkeys[DXKEY_PAD8]='8';
	dxkeys[DXKEY_PAD9]='9';
	dxkeys[DXKEY_PAD0]='0';
	dxkeys[DXKEY_MINUS]='-';
	dxkeys[DXKEY_SPACE]=' ';
	dxkeys[DXKEY_FSLASH]='/';
	dxkeys[DXKEY_PAD_SLASH]='/';
	dxkeys[DXKEY_BSLASH]='\\';
	dxkeys[DXKEY_BSLASH2]='\\';
	dxkeys[DXKEY_PERIOD]='.';
}
